@@ -3,7 +3,7 @@ package SVN::Look;
 use strict;
 use warnings;
 use Carp;
-use File::Spec::Functions qw/catfile path rootdir/;
+use File::Spec::Functions;
 
 =head1 NAME
 
@@ -11,11 +11,11 @@ SVN::Look - A caching wrapper aroung the svnlook command.
 
 =head1 VERSION
 
-Version 0.18
+Version 0.22
 
 =cut
 
-our $VERSION = '0.18';
+our $VERSION = '0.22';
 
 =head1 SYNOPSIS
 
@@ -43,22 +43,21 @@ in the object, avoiding repetitious calls.
 
 =cut
 
-our $SVNLOOK;
-my $root = rootdir();
-for my $d (
-    path(),
-    catfile($root, 'usr', 'local', 'bin'),
-    catfile($root, 'usr', 'bin'),
-    catfile($root, 'bin'),
-) {
-    my $f = catfile($d, 'svnlook');
-    if (-x $f) {
-        $SVNLOOK = $f;
-        last;
+BEGIN {
+    if ($^O eq 'MSWin32') {
+	$ENV{PATH} .= ';C:\Program Files (x86)\VisualSVN Server\bin';
+    } else {
+	$ENV{PATH} .= ':/usr/local/bin:/usr/bin:/bin';
     }
+    eval {
+	open my $pipe, '-|', "svnlook --version" or die;
+	local $/ = undef;		# slurp mode
+	<$pipe>;
+	close $pipe or die;
+    };
+    die "Aborting because I couldn't execute the svnlook command: $@\n"
+	if $@;
 }
-die "Aborting because I couldn't find the svnlook executable.\n"
-    unless $SVNLOOK;
 
 =head1 METHODS
 
@@ -109,9 +108,9 @@ sub new {
 
 sub _svnlook {
     my ($self, $cmd, @args) = @_;
-    my @cmd = ($SVNLOOK, $cmd, $self->{repo});
+    my @cmd = (svnlook => $cmd, $self->{repo});
     push @cmd, @{$self->{what}} unless $cmd =~ /^(?:youngest|uuid|lock)$/;
-    open my $fd, '-|', @cmd, @args
+    open my $fd, '-|', '"' . join('"  "', @cmd, @args) . '"'
         or die "Can't exec svnlook $cmd: $!\n";
     if (wantarray) {
         my @lines = <$fd>;
@@ -227,7 +226,7 @@ sub proplist {
 
 Returns a reference to a hash containing information about all file
 changes occurred in the revision. The hash always has the following
-pairs:
+keys:
 
 =over
 
@@ -249,8 +248,7 @@ A list of files that had properties modified in the revision.
 
 =item copied
 
-A hash mapping the old to the new name of each file moved in the
-revision.
+A hash containing information about each file or diretory copied in the revision. The hash keys are the names of elements copied to. The value associated with a key is a two-element array containing the name of the element copied from and the specific revision from which it was copied.
 
 =back
 
@@ -369,29 +367,29 @@ sub dirs_changed {
     return @{$self->{dirs_changed}};
 }
 
-=item B<copied_from>
-
-Returns the list of original names of files that were renamed in the
-revision/transaction.
-
-=cut
-
-sub copied_from {
-    my $self = shift;
-    return keys %{$self->changed_hash()->{copied_from}};
-}
-
 =item B<copied_to>
 
-Returns the list of new names of files that were renamed in the
-revision/transaction. The order of this list is guaranteed to agree
-with the order generated by the method copied_from.
+Returns the list of new names of files that were copied in the
+revision/transaction.
 
 =cut
 
 sub copied_to {
     my $self = shift;
-    return values %{$self->changed_hash()->{copied_from}};
+    return keys %{$self->changed_hash()->{copied}};
+}
+
+=item B<copied_from>
+
+Returns the list of original names of files that were copied in the
+revision/transaction. The order of this list is guaranteed to agree
+with the order generated by the method copied_to.
+
+=cut
+
+sub copied_from {
+    my $self = shift;
+    return map {$_->[0]} values %{$self->changed_hash()->{copied}};
 }
 
 =item B<cat> PATH
@@ -508,6 +506,35 @@ sub lock {
     return %lock ? \%lock : undef;
 }
 
+=item B<tree> [PATH_IN_REPOS, OPTS, ...]
+
+Returns the repository tree as a list of paths, starting at
+PATH_IN_REPOS (if supplied, at the root of the tree otherwise),
+optionally showing node revision ids.
+
+=over
+
+=item C<--full-paths>
+
+show full paths instead of indenting them.
+
+=item C<--show-ids>
+
+Returns the node revision ids for each path.
+
+=item C<--non-recursive>
+
+Operate on single directory only.
+
+=back
+
+=cut
+
+sub tree {
+    my ($self, @opts) = @_;
+    return $self->_svnlook('tree', @opts);
+}
+
 =back
 
 =head1 AUTHOR
@@ -550,7 +577,7 @@ L<http://search.cpan.org/dist/SVN-Hooks>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2010 CPqD, all rights reserved.
+Copyright 2008-2011 CPqD, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
